package fr.im2ag.m2cci.panier;

import java.util.Iterator;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;

import static org.junit.jupiter.api.Assertions.*;

/**
 * Tests unitaires de la classe Panier
 * @author Philippe Genoud - LIG Steamer - Université Grenoble Aples
 */
public class PanierTest {

    private Panier panier = null;

    private final Produit p1 = new Produit(1234, "PRODUIT1", "description du produit 1", 100, 5);
    private final Produit p2 = new Produit(2234, "PRODUIT2", "description du produit 2", 20, 20);
    private final Produit p3 = new Produit(3234, "PRODUIT3", "description du produit 3", 50, 5);

    public PanierTest() {
    }

    /**
     * methode exécutée avant chaque test.
     *
     * crée un panier avec 3 articles :
     *
     * - le produit p1 en deux exemplaires
     * 
     * - le produit p2 en un exemplaire
     *
     * - le produit p3 en trois exemplaires
     */
    @BeforeEach
    public void setUp() {
        panier = new Panier();
        panier.ajouter(p1, 2);
        panier.ajouter(p2, 1);
        panier.ajouter(p3, 3);
    }

    /**
     * Test of getNbArticles method, of class Panier.
     */
    @Test
    public void testGetNbArticles() {
        System.out.println("getNbArticles");
        assertEquals(3, panier.getNbArticles());
    }

    /**
     * Test of getArticle method, of class Panier.
     */
    @Test
    public void testArticle() {
        System.out.println("geArticle");
        ArticlePanier article = panier.getArticle(p2.getCode());
        assertEquals(1, article.getQuantite());
        assertEquals(p2, article.getProduit());
    }

    /**
     * vérifie que lorsque l'on ajoute un nouveau produit au panier un nouvel
     * article est bien créé
     */
    @Test
    public void testAjouter1() {
        System.out.println("ajouter1");
        panier.ajouter(new Produit(4234, "PRODUIT4", "description du produit 4", 60, 0.2), 1);
        assertEquals(4, panier.getNbArticles());
    }

    /**
     * vérifie que lorsque l'on ajoute un produit pour lequel il existe déjà un
     * article aucun nouvel article n'est crée et que la quantité est bien
     * modifiée
     */
    @Test
    public void testAjouter2() {
        System.out.println("ajouter2");
        panier.ajouter(p3, 1);
        assertEquals(3, panier.getNbArticles());
        assertEquals(4, panier.getArticle(3234).getQuantite());
    }

    /**
     * vérifie que l'on ne peut pas ajouter un produit au panier avec une quantité négative
     */
    @Test
    public void testAjouter3() {
        System.out.println("ajouter3");
        assertThrows(IllegalArgumentException.class,  () -> panier.ajouter(p3, -1));
    }

    /**
     * verifie que augmenterQuantité lance bien une exception si le code du
     * produit ne correspond à aucun article
     */
    @Test
    public void testAugmenterQuantite1() {
        assertThrows(ProduitNonPresentException.class,
                () -> panier.augmenterQuantite(1555, 4));
    }

    /**
     * verifie que augmenterQuantité lance bien une exception
     * IllegalArgumentException si la quantité spécifiée est <= 0
     */
    @Test
    public void testAugmenterQuantite2() {
        assertThrows(IllegalArgumentException.class,
                () -> {
                    panier.augmenterQuantite(2234, -2);
                });
    }

    /**
     * Test of augmenterQuantite method, of class Panier.
     * 
     * @throws ProduitNonPresentException
     */
    @Test
    public void testAugmenterQuantite3() throws ProduitNonPresentException {
        System.out.println("augmenterQuantite3");
        panier.augmenterQuantite(2234, 2);
        assertEquals(3, panier.getArticle(2234).getQuantite());
    }

    /**
     * verifie que diminuerQuantité lance bien une exception
     * ProduitNonPresentException si le code du produit ne correspond à aucun
     * article
     */
    @Test
    public void testDiminuerQuantite1() {
        assertThrows(ProduitNonPresentException.class,
                () -> panier.diminuerQuantite(1555, 4));
    }

    /**
     * verifie que diminuerQuantité lance bien une exception
     * IllegalArgumentExceptions si la quantité spécifiée est <= 0
     */
    @Test
    public void testDiminuerQuantite2() {
        assertThrows(IllegalArgumentException.class,
                () -> panier.diminuerQuantite(2234, -2));
    }

    /**
     * Test of diminuerQuantite method, of class Panier.
     *
     * @throws fr.im2ag.m2cci.panier.ProduitNonPresentException
     */
    @Test
    public void testDiminuerQuantite3() throws ProduitNonPresentException {
        System.out.println("diminuerQuantite");
        panier.diminuerQuantite(p3.getCode(), 2);
        assertEquals(1, panier.getArticle(p3.getCode()).getQuantite());
    }

    /**
     * Vérifie qu'un article est bien supprimé du panier si la valeur utilisée
     * pour diminuer sa quantité est supérieure ou égale à la quantité présente
     * 
     * throws ProduitNonPresentException
     */
    @Test
    public void testDiminuerQuantite4() throws ProduitNonPresentException {
        System.out.println("diminuerQuantite");
        panier.diminuerQuantite(p3.getCode(), 3);
        assertEquals(2, panier.getNbArticles());
        assertNull(panier.getArticle(p3.getCode()));

        panier.diminuerQuantite(p1.getCode(), 12);

        assertEquals(1, panier.getNbArticles());
        assertNull(panier.getArticle(p1.getCode()));
    }

    /**
     * Test of vider method, of class Panier.
     */
    @Test
    public void testVider() {
        System.out.println("vider");
        panier.vider();
        assertEquals(0, panier.getNbArticles());
    }

    /**
     * Verifie qu'après la suppression d'un article le nombre d'articles du
     * panier a bien diminué de 1 et que l'article supprimé n'est plus présent.
     */
    @Test
    public void testSupprimer() {
        System.out.println("supprimer");
        panier.supprimer(p2.getCode());
        assertEquals(2, panier.getNbArticles());
        assertNull(panier.getArticle(p2.getCode()));
    }

    /**
     * Test of getPrixTotalHT method, of class Panier.
     */
    @Test
    public void testGetPrixTotalHT() {
        System.out.println("getPrixTotalHT");
        assertEquals(370.0, panier.getPrixTotalHT());
    }

    /**
     * Test of getPrixTotalTTC method, of class Panier.
     */
    @Test
    public void testGetPrixTotalTTC() {
        System.out.println("getPrixTotalTTC");
        assertEquals(391.5, panier.getPrixTotalTTC());
    }

    /**
     * Test of iterator method, of class Panier.
     */
    @Test
    public void testIterator() {
        System.out.println("iterator");
        Iterator<ArticlePanier> it = panier.iterator();
        int nbArticles = 0;
        int quantitéTotale = 0;
        boolean presentP1 = false;
        boolean presentP2 = false;
        boolean presentP3 = false;
        while (it.hasNext()) {
            ArticlePanier article = it.next();
            nbArticles++;
            quantitéTotale += article.getQuantite();
            switch (article.getProduit().getCode()) {
                case 1234:
                    presentP1 = true;
                    break;
                case 2234:
                    presentP2 = true;
                    break;
                case 3234:
                    presentP3 = true;
                    break;
                default:
                    fail("Code d'article innatendu " + article.getProduit().getCode());
            }
        }
        assertEquals(3, nbArticles);
        assertEquals(6, quantitéTotale);
        assertTrue(presentP1);
        assertTrue(presentP2);
        assertTrue(presentP3);
    }
}
